/*
 *  Gary Jennejohn (C) 2003 <gj@denx.de>
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * Routines for generic manipulation of the interrupts found on the 
 * INCAIP boards.
 */
#include <linux/config.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/interrupt.h>
#include <linux/kernel_stat.h>

#include <asm/bootinfo.h>
#include <asm/irq.h>
#include <asm/incaip/inca-ip.h>
#include <asm/incaip/irq.h>
#if 0
#include <asm/gdb-stub.h>
#endif


extern asmlinkage void mipsIRQ(void);
extern void do_IRQ(int irq, struct pt_regs *regs);

#if 0
#define DEBUG_INT(x...) printk(x)
#else
#define DEBUG_INT(x...)
#endif

void disable_incaip_irq(unsigned int irq_nr)
{
	DEBUG_INT("disable_incaip_irq: %d ", irq_nr);
	/* have to access the correct register here */
	if (irq_nr <= INT_NUM_IM0_IRL31 && irq_nr >= INT_NUM_IM0_IRL0)
	{
		/* access IM0 */
		*INCA_IP_ICU_IM0_IER &= ~INCA_IP_ICU_IM0_IER_IR(irq_nr);
	}
	if (irq_nr <= INT_NUM_IM1_IRL31 && irq_nr >= INT_NUM_IM1_IRL0)
	{
		/* access IM1 */
		*INCA_IP_ICU_IM1_IER &= ~INCA_IP_ICU_IM1_IER_IR(irq_nr - 32);
	}
	if (irq_nr <= INT_NUM_IM2_IRL31 && irq_nr >= INT_NUM_IM2_IRL0)
	{
		/* access IM2 */
		*INCA_IP_ICU_IM2_IER &= ~INCA_IP_ICU_IM2_IER_IR(irq_nr - 64);
	}
}

void mask_and_ack_incaip_irq(unsigned int irq_nr)
{
	DEBUG_INT("mask_and_ack_incaip_irq: %d ", irq_nr);
	/* have to access the correct register here */
	if (irq_nr <= INT_NUM_IM0_IRL31 && irq_nr >= INT_NUM_IM0_IRL0)
	{
		/* access IM0 */
		/* mask */
		*INCA_IP_ICU_IM0_IER &= ~INCA_IP_ICU_IM0_IER_IR(irq_nr);
		/* ack */
		*INCA_IP_ICU_IM0_ISR = INCA_IP_ICU_IM0_ISR_IR(irq_nr);
	}
	if (irq_nr <= INT_NUM_IM1_IRL31 && irq_nr >= INT_NUM_IM1_IRL0)
	{
		/* access IM1 */
		/* mask */
		*INCA_IP_ICU_IM1_IER &= ~INCA_IP_ICU_IM1_IER_IR(irq_nr - 32);
		/* ack */
		*INCA_IP_ICU_IM1_ISR = INCA_IP_ICU_IM1_ISR_IR(irq_nr - 32);
	}
	if (irq_nr <= INT_NUM_IM2_IRL31 && irq_nr >= INT_NUM_IM2_IRL0)
	{
		/* access IM2 */
		/* mask */
		*INCA_IP_ICU_IM2_IER &= ~INCA_IP_ICU_IM2_IER_IR(irq_nr - 64);
		/* ack */
		*INCA_IP_ICU_IM2_ISR = INCA_IP_ICU_IM2_ISR_IR(irq_nr - 64);
	}
}

void enable_incaip_irq(unsigned int irq_nr)
{
	DEBUG_INT("enable_incaip_irq: %d ", irq_nr);
	/* have to access the correct register here */
	if (irq_nr <= INT_NUM_IM0_IRL31 && irq_nr >= INT_NUM_IM0_IRL0)
	{
		/* access IM0 */
		*INCA_IP_ICU_IM0_IER |= INCA_IP_ICU_IM0_IER_IR(irq_nr);
	}
	if (irq_nr <= INT_NUM_IM1_IRL31 && irq_nr >= INT_NUM_IM1_IRL0)
	{
		/* access IM1 */
		*INCA_IP_ICU_IM1_IER |= INCA_IP_ICU_IM1_IER_IR(irq_nr - 32);
	}
	if (irq_nr <= INT_NUM_IM2_IRL31 && irq_nr >= INT_NUM_IM2_IRL0)
	{
		/* access IM2 */
		*INCA_IP_ICU_IM2_IER |= INCA_IP_ICU_IM2_IER_IR(irq_nr - 64);
	}
}

static unsigned int startup_incaip_irq(unsigned int irq)
{
	enable_incaip_irq(irq);
	return 0; /* never anything pending */
}

#define shutdown_incaip_irq	disable_incaip_irq

static void end_incaip_irq(unsigned int irq)
{
	if (!(irq_desc[irq].status & (IRQ_DISABLED|IRQ_INPROGRESS)))
		enable_incaip_irq(irq);
}

static struct hw_interrupt_type incaip_irq_type = {
	"INCAIP",
	startup_incaip_irq,
	shutdown_incaip_irq,
	enable_incaip_irq,
	disable_incaip_irq,
	mask_and_ack_incaip_irq,
	end_incaip_irq,
	NULL
};

/* find the least signifcant bit set in x */
static inline int ls1bit32(unsigned int x)
{
	int b = 31, s;

	s = 16; if (x << 16 == 0) s = 0; b -= s; x <<= s;
	s =  8; if (x <<  8 == 0) s = 0; b -= s; x <<= s;
	s =  4; if (x <<  4 == 0) s = 0; b -= s; x <<= s;
	s =  2; if (x <<  2 == 0) s = 0; b -= s; x <<= s;
	s =  1; if (x <<  1 == 0) s = 0; b -= s;

	return b;
}

/* Cascaded interrupts from IM0 */
void incaip_hw0_irqdispatch(struct pt_regs *regs)
{
	struct irqaction *action;
	unsigned long int_status;
	int irq;

	int_status = *INCA_IP_ICU_IM0_IOSR;

	/* if int_status == 0, then the interrupt has already been cleared */
	if (int_status == 0)
		return;

	irq = ls1bit32(int_status);

	DEBUG_INT("incaip_hw0_irqdispatch: irq=%d\n", irq);

#if 0
	action = irq_desc[irq].action;
	/* if action == NULL, then we don't have a handler for the irq */
	if (action == NULL) {
		printk("No handler for hw0 irq: %i\n", irq);
		atomic_inc(&irq_err_count);
		return;
	}

	irq_enter();
	kstat_cpu(0).irqs[irq]++;
	action->handler(irq, action->dev_id, regs);
	irq_exit();
#else
	DEBUG_INT("hw0 ");
	do_IRQ(irq, regs);
#endif

	return;
}

/* Cascaded interrupts from IM1 */
void incaip_hw1_irqdispatch(struct pt_regs *regs)
{
	struct irqaction *action;
	unsigned long int_status;
	int irq;

	int_status = *INCA_IP_ICU_IM1_IOSR;

	/* if int_status == 0, then the interrupt has already been cleared */
	if (int_status == 0)
		return;

	irq = ls1bit32(int_status) + 32;

	DEBUG_INT("incaip_hw1_irqdispatch: irq=%d\n", irq);

#if 0
	action = irq_desc[irq].action;
	/* if action == NULL, then we don't have a handler for the irq */
	if (action == NULL) {
		printk("No handler for hw1 irq: %i\n", irq);
		atomic_inc(&irq_err_count);
		return;
	}

	irq_enter();
	kstat_cpu(0).irqs[irq]++;
	action->handler(irq, action->dev_id, regs);
	irq_exit();
#else
	DEBUG_INT("hw1 ");
	do_IRQ(irq, regs);
#endif

	return;
}

/* Cascaded interrupts from IM2 */
void incaip_hw2_irqdispatch(struct pt_regs *regs)
{
	struct irqaction *action;
	unsigned long int_status;
	int irq;

	int_status = *INCA_IP_ICU_IM2_IOSR;

	/* if int_status == 0, then the interrupt has already been cleared */
	if (int_status == 0)
		return;

	irq = ls1bit32(int_status) + 64;

	DEBUG_INT("incaip_hw2_irqdispatch: irq=%d\n", irq);

#if 0
	action = irq_desc[irq].action;
	/* if action == NULL, then we don't have a handler for the irq */
	if (action == NULL) {
		printk("No handler for hw2 irq: %i\n", irq);
		atomic_inc(&irq_err_count);
		return;
	}

	irq_enter();
	kstat_cpu(0).irqs[irq]++;
	action->handler(irq, action->dev_id, regs);
	irq_exit();
#else
	DEBUG_INT("hw2 ");
	do_IRQ(irq, regs);
#endif

	return;
}

/* Function for careful CP0 interrupt mask access */

void __init init_IRQ(void)
{
	int i;

	DEBUG_INT("init_IRQ ");

	init_generic_irq();

	/* mask all interrupt sources */
	*INCA_IP_ICU_IM0_IER = 0;
	*INCA_IP_ICU_IM1_IER = 0;
	*INCA_IP_ICU_IM2_IER = 0;

	/* Now safe to set the exception vector. */
	set_except_vector(0, mipsIRQ);

	for (i = 0; i <= INT_NUM_IM2_IRL31; i++) {
		irq_desc[i].status	= IRQ_DISABLED;
		irq_desc[i].action	= 0;
		irq_desc[i].depth	= 1;
		irq_desc[i].handler	= &incaip_irq_type;
	}
	/* VERY important! */
#ifdef COUNTER_WORKS
	set_cp0_status(IE_IRQ0 | IE_IRQ1 | IE_IRQ2 | IE_IRQ3 | IE_IRQ4 | IE_IRQ5);
#else
	clear_cp0_status(IE_IRQ5);
	set_cp0_status(IE_IRQ0 | IE_IRQ1 | IE_IRQ2 | IE_IRQ3 | IE_IRQ4);
#endif
}
