/*
 * setup.c
 *
 * Gary Jennejohn (C) 2003 <gj@denx.de>
 *
 * ########################################################################
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * ########################################################################
 *
 * INCAIP specific setup.
 */
#include <linux/config.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/pci.h>
#include <linux/ide.h>

#include <linux/interrupt.h>
#include <asm/time.h>

#include <asm/cpu.h>
#include <asm/bootinfo.h>
#include <asm/irq.h>
#include <asm/incaip/inca-ip.h>
#include <asm/incaip/irq.h>

#if 0
extern char arcs_cmdline[CL_SIZE];
#endif

extern void incaip_reboot_setup(void);

void __init bus_error_init(void) { /* nothing */ }

/* the CPU clock rate - lifted from u-boot */
unsigned int
incaip_get_cpu_hz(void)
{
	/*----------------------------------------------------------------*/
	/* CPU Clock Input Multiplexer (MUX I)                            */
	/* Multiplexer MUX I selects the maximum input clock to the CPU.  */
	/*----------------------------------------------------------------*/
	if (*INCA_IP_CGU_CGU_MUXCR & INCA_IP_CGU_CGU_MUXCR_MUXI)
	{
		/* MUX I set to 150 MHz clock */
		return 150000000;
	}
	else
	{
		/* MUX I set to 100/133 MHz clock */
		if (*INCA_IP_CGU_CGU_DIVCR & 0x40) 
		{
			 /* Division value is 1/3, maximum CPU operating */
			 /* frequency is 133.3 MHz                       */
			 return 133333333;
		}
		else
		{
			 /* Division value is 1/4, maximum CPU operating */
			 /* frequency is 100 MHz                         */
			 return 100000000;
		}
	}
}

/* the FPI clock rate - lifted from u-boot */
unsigned int
incaip_get_fpi_hz(void)
{
	unsigned int  clkCPU;

	clkCPU = incaip_get_cpu_hz();

	switch (*INCA_IP_CGU_CGU_DIVCR & 0xC)
	{
		case 0x4:
			return clkCPU >> 1; /* divided by 2 */
		break;
		case 0x8:
			return clkCPU >> 2; /* divided by 4 */
		break;
		default:
			return clkCPU;
		break;
	}
}

void incaip_time_init()
{
	extern int *cpuoptions;

	/* do not want to set mips_counter_frequency! */
	/* I don't know whether this is kosher, but CPU_COUNTER is broken */
	*cpuoptions &= ~MIPS_CPU_COUNTER;
}

/* this gets called indirectly from incaip_hw2_irqdispatch */
void incaip_timer6_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
#ifndef COUNTER_WORKS
/*printk("T ");*/
	timer_interrupt(INT_NUM_IM2_IRL29, NULL, regs);
#endif
}

/*
 * THe CPU timer doesn't work (yet) on INCAIP, so we have to
 * use TIMER6 in its place. Takenb from sysLib.c. Timer is set
 * to 100 Hz.
 */
void incaip_timer_setup(struct irqaction *irq)
{
#ifndef COUNTER_WORKS
	volatile unsigned int *pPM_CR = INCA_IP_PMU_PM_CR;
	volatile unsigned int *pPM_GEN = INCA_IP_PMU_PM_GEN;
	volatile unsigned int *pCGU_DIVCR = INCA_IP_CGU_CGU_DIVCR;
	int retval;

	/* setup the PMU */
	*pPM_CR =  (*pPM_CR & ~3) | INCA_IP_PMU_PM_CR_SWCR | INCA_IP_PMU_PM_CR_SWRST;

	/* setup the CGU */
	*pCGU_DIVCR |= (1 << 31) | (1 << 2);

	/* enable the timer in the PMU */
	*pPM_GEN |= INCA_IP_PMU_PM_GEN_EN10;

	/* setup the GPTU for timer tick */
	*(INCA_IP_GPTU_GPT_CLC) = 0x100;

	if (incaip_get_cpu_hz() == 100000000)
	{
printk("100MHz\n");
		/* for CPU frequency is 100 MHz */
		*(INCA_IP_GPTU_GPT_CAPREL) = 0x3D09;
		*(INCA_IP_GPTU_GPT_T6CON) = 0x80C3;
	}
	else if (incaip_get_cpu_hz() == 150000000)
	{
printk("150MHz\n");
		/* for CPU frequency is 150 MHz */
		*(INCA_IP_GPTU_GPT_CAPREL) = 0xB71B;
		*(INCA_IP_GPTU_GPT_T6CON) = 0x80C2;
	}
	else
	{
printk("133MHz\n");
		/* for CPU frequency of 133 MHz */
		*(INCA_IP_GPTU_GPT_CAPREL) = 0xA2C2;
		*(INCA_IP_GPTU_GPT_T6CON) = 0x80C2;
	}

	/* do it by hand */
	irq->handler = incaip_timer6_interrupt;
	retval = setup_irq(INT_NUM_IM2_IRL29, irq);
	if (retval)
		printk("setup_irq failed\n");
#endif
}

#define MIPS_CPU_TIMER_IRQ 7
/* this gets called straight from incaipIRQ.S */
asmlinkage void incaip_timer_interrupt(struct pt_regs *regs)
{
#ifdef COUNTER_WORKS
	ll_timer_interrupt(MIPS_CPU_TIMER_IRQ, regs);
#endif
}

void __init incaip_setup(void)
{
	incaip_reboot_setup();
	board_time_init = incaip_time_init;
	board_timer_setup = incaip_timer_setup;

#if 0
	/* Switch from prom exception handler to normal mode */
	change_c0_status(ST0_BEV,0);

	prom_printf("platform_setup done\n");
#endif
}

#if 0 /* reference */
unsigned long lasat_flash_partition_start(int partno)
{
	unsigned long dst;

	switch (partno) {
	case LASAT_MTD_BOOTLOADER:
		dst = lasat_board_info.li_flash_service_base;
		break;
	case LASAT_MTD_SERVICE:
		dst = lasat_board_info.li_flash_service_base + BOOTLOADER_SIZE;
		break;
	case LASAT_MTD_NORMAL:
		dst = lasat_board_info.li_flash_normal_base;
		break;
	case LASAT_MTD_FS:
		dst = lasat_board_info.li_flash_fs_base;
		break;
	case LASAT_MTD_CONFIG:
		dst = lasat_board_info.li_flash_cfg_base;
		break;
	default:
		dst = 0;
		break;
	}

	return dst;
}

unsigned long lasat_flash_partition_size(int partno)
{
	unsigned long size;

	switch (partno) {
	case LASAT_MTD_BOOTLOADER:
		size = BOOTLOADER_SIZE;
		break;
	case LASAT_MTD_SERVICE:
		size = lasat_board_info.li_flash_service_size - BOOTLOADER_SIZE;
		break;
	case LASAT_MTD_NORMAL:
		size = lasat_board_info.li_flash_normal_size;
		break;
	case LASAT_MTD_FS:
		size = lasat_board_info.li_flash_fs_size;
		break;
	case LASAT_MTD_CONFIG:
		size = lasat_board_info.li_flash_cfg_size;
		break;
	default:
		size = 0;
		break;
	}

	return size;
}
#endif
