/*
 * Header file for DMA on the INCA.
 * Taken from the VxWorks sources by Infineon.
 */

#ifndef _INCADMA_H_
#define _INCADMA_H_

/* Configuration macros */
#define INCA_DMA_USE_COPYING
/*#define DMA_DEBUG_ENABLE*/

#define INCA_DMA_TX_POLLING_TIME       0x07
#define INCA_DMA_RX_POLLING_TIME       0x07
#define INCA_DMA_MAX_TX_CHANNEL_ID     0x02
#define INCA_DMA_MAX_RX_CHANNEL_ID     0x01
#define INCA_DMA_MAX_TX_PRIORITY       0x03

#define INCA_IP_DMA_TXIMR_RDERR(x)     (1 << (12+x))
#define INCA_IP_DMA_TXIMR_HLD(x)       (1 << (9+x))
#define INCA_IP_DMA_TXIMR_CPT(x)       (1 << (6+x))
#define INCA_IP_DMA_TXIMR_EOP(x)       (1 << (3+x))
#define INCA_IP_DMA_TXIMR_CMDCPT(x)    (1 << x)

#define INCA_IP_DMA_TXISR_RDERR(x)     (1 << (12+x))
#define INCA_IP_DMA_TXISR_HLD(x)       (1 << (9+x))
#define INCA_IP_DMA_TXISR_CPT(x)       (1 << (6+x))
#define INCA_IP_DMA_TXISR_EOP(x)       (1 << (3+x))
#define INCA_IP_DMA_TXISR_CMDCPT(x)    (1 << x)

#define INCA_IP_DMA_RXIMR_RDERR(x)     (1 << (8+x))
#define INCA_IP_DMA_RXIMR_CMDCPT(x)    (1 << (6+x))
#define INCA_IP_DMA_RXIMR_EOP(x)       (1 << (4+x))
#define INCA_IP_DMA_RXIMR_CPT(x)       (1 << (2+x))
#define INCA_IP_DMA_RXIMR_HLD(x)       (1 << x)

#define INCA_IP_DMA_RXISR_RDERR(x)     (1 << (8+x))
#define INCA_IP_DMA_RXISR_CMDCPT(x)    (1 << (6+x))
#define INCA_IP_DMA_RXISR_EOP(x)       (1 << (4+x))
#define INCA_IP_DMA_RXISR_CPT(x)       (1 << (2+x))
#define INCA_IP_DMA_RXISR_HLD(x)       (1 << x)

#define INCA_IP_DMA_TXWGT_TXPRI(x,value)   ((value & 0x3) << (x*2))

#define INCA_IP_DMA_RXISR_HLD_0        0x00000001
#define INCA_IP_DMA_RXISR_HLD_1        0x00000002
#define INCA_IP_DMA_RXISR_CPT_0        0x00000004
#define INCA_IP_DMA_RXISR_CPT_1        0x00000008
#define INCA_IP_DMA_RXISR_EOP_0        0x00000010
#define INCA_IP_DMA_RXISR_EOP_1        0x00000020
#define INCA_IP_DMA_RXISR_CMDCPT_0     0x00000040
#define INCA_IP_DMA_RXISR_CMDCPT_1     0x00000080
#define INCA_IP_DMA_RXISR_RDERR_0      0x00000100
#define INCA_IP_DMA_RXISR_RDERR_1      0x00000200

#define INCA_IP_DMA_RXISR_HLD_0        0x00000001
#define INCA_IP_DMA_RXISR_HLD_1        0x00000002
#define INCA_IP_DMA_RXISR_CPT_0        0x00000004
#define INCA_IP_DMA_RXISR_CPT_1        0x00000008
#define INCA_IP_DMA_RXISR_EOP_0        0x00000010
#define INCA_IP_DMA_RXISR_EOP_1        0x00000020
#define INCA_IP_DMA_RXISR_CMDCPT_0     0x00000040
#define INCA_IP_DMA_RXISR_CMDCPT_1     0x00000080
#define INCA_IP_DMA_RXISR_RDERR_0      0x00000100
#define INCA_IP_DMA_RXISR_RDERR_1      0x00000200

#define INCA_IP_DMA_TXISR_CMDCPT_0     0x00000001
#define INCA_IP_DMA_TXISR_CMDCPT_1     0x00000002
#define INCA_IP_DMA_TXISR_CMDCPT_2     0x00000004
#define INCA_IP_DMA_TXISR_EOP_0        0x00000008
#define INCA_IP_DMA_TXISR_EOP_1        0x00000010
#define INCA_IP_DMA_TXISR_EOP_2        0x00000020
#define INCA_IP_DMA_TXISR_CPT_0        0x00000040
#define INCA_IP_DMA_TXISR_CPT_1        0x00000080
#define INCA_IP_DMA_TXISR_CPT_2        0x00000100
#define INCA_IP_DMA_TXISR_HLD_0        0x00000200
#define INCA_IP_DMA_TXISR_HLD_1        0x00000400
#define INCA_IP_DMA_TXISR_HLD_2        0x00000800
#define INCA_IP_DMA_TXISR_RDERR_0      0x00001000
#define INCA_IP_DMA_TXISR_RDERR_1      0x00002000
#define INCA_IP_DMA_TXISR_RDERR_2      0x00004000

#define INCA_IRQ_LINE_DMAALL   0x18000000  /* all IRQs */
#define INCA_IRQ_LINE_DMATIR   0x08000000  /* TIR - Tx */
#define INCA_IRQ_LINE_DMARIR   0x10000000  /* RIR - Rx */


#define DMA_INTERRUPTS_ENABLE(X)  *((volatile unsigned long*) INCA_IP_ICU_IM1_IER) |= X;
#define DMA_INTERRUPTS_DISABLE(X) *((volatile unsigned long*) INCA_IP_ICU_IM1_IER) &= ~X;
#define DMA_INTERRUPTS_CLEAR(X)   *((volatile unsigned long*) INCA_IP_ICU_IM1_ISR) = X;

typedef struct
{
   /* first DWORD */
   union
   {
      struct
      {
         volatile unsigned long HOLD                :1;
         volatile unsigned long ICpt                :1;
         volatile unsigned long IEop                :1;
         volatile unsigned long offset              :3;
         volatile unsigned long reserved0           :4;
         volatile unsigned long NFB                 :22;
      } field;

      volatile unsigned long dword;
   } params;

   /* second DWORD */
   volatile unsigned long nextRxDescPtr;

   /* third DWORD */
   volatile unsigned long RxDataPtr;

   /* fourth DWORD */
   union
   {
      struct
      {
         volatile unsigned long C                   :1;
         volatile unsigned long Sop                 :1;
         volatile unsigned long Eop                 :1;
         volatile unsigned long reserved3           :12;
         volatile unsigned long NBT                 :17;
      } field;
      volatile unsigned long dword;
   } status;

} IncaDmaRxDescriptor_t;

#define INCA_DMA_RX_C   0x80000000
#define INCA_DMA_RX_SOP 0x40000000
#define INCA_DMA_RX_EOP 0x20000000

typedef struct
{
   /* first DWORD */
   union
   {
      struct
      {
         volatile unsigned long HOLD                :1;
         volatile unsigned long Eop                 :1;
         volatile unsigned long Sop                 :1;
         volatile unsigned long ICpt                :1;
         volatile unsigned long IEop                :1;
         volatile unsigned long reserved0           :5;
         volatile unsigned long NBA                 :22;
      } field;

      volatile unsigned long dword;
   } params;

   /* second DWORD */
   volatile unsigned long nextTxDescPtr;

   /* third DWORD */
   volatile unsigned long TxDataPtr;

   /* fourth DWORD */
   volatile unsigned long C                   :1;
   volatile unsigned long reserved3           :31;

} IncaDmaTxDescriptor_t;

#define INCA_DMA_TX_HOLD   0x80000000
#define INCA_DMA_TX_EOP    0x40000000
#define INCA_DMA_TX_SOP    0x20000000
#define INCA_DMA_TX_ICPT   0x10000000
#define INCA_DMA_TX_IEOP   0x08000000


typedef struct
{
   void                    *firstDescriptorAddr;
   IncaDmaRxDescriptor_t   *holdDescriptor;
   IncaDmaRxDescriptor_t   *currentDescriptor;
   IncaDmaRxDescriptor_t   *firstDescriptor;
   unsigned long                  descriptorNumber;
   unsigned long                  bufferSize;
   unsigned long                  (*getNewBuffer)(void *device);
   unsigned long                  (*putRcvdBuffer)(void *device, void *buf, unsigned long bufSize);
   void                    *device;
   unsigned long                  rxCounter;
   unsigned long                  maxRxInOneIrq;
} IncaDmaRxChannel_t;

typedef struct
{
   void                    *firstDescriptorAddr;
   IncaDmaTxDescriptor_t   *holdDescriptor;
   IncaDmaTxDescriptor_t   *currentDescriptor;
   unsigned long                  descriptorNumber;
   unsigned long                  (*txComplete)(void *device);
   void                    *device;
   int                     bInit;

   unsigned char                   *currentBufferPtr;
   unsigned long                  curPacketSize;

   unsigned long                  txCounter;
   unsigned long                  useCounter;
   unsigned long                  txAckCounter;
   unsigned long                  maxPacketSize;

} IncaDmaTxChannel_t;

int incaDMA_Init (void);

long incaDMA_InitRxChannel(   unsigned long  channelId, 
                                       unsigned long descNum, 
                                       unsigned long bufSize, 
                                       void *device, 
                                       unsigned long (*getNewBuffer)(void *device),
                                       unsigned long (*putRcvdBuffer)(void *device, void *buf, unsigned long bufSize) );

long incaDMA_InitTxChannel(   unsigned long  channelId, 
                                       unsigned long descNum, 
                                       unsigned long priority, 
                                       void *device, 
                                       unsigned long (*txComplete)(void *device) );

unsigned long incaDMA_Transmit    (   unsigned long  channelId, 
                                       void *buffer, 
                                       unsigned long bufLen, 
                                       int lastBuffer );


long incaDMA_CloseTxChannel ( unsigned long channelId );
long incaDMA_CloseRxChannel ( unsigned long channelId ); 

#ifdef DMA_DEBUG_ENABLE
void incaDMA_PrintAllInfo(unsigned long channelId);
void incaDMA_PrintTxInfo( IncaDmaTxDescriptor_t *desc );
void incaDMA_PrintRxInfo( IncaDmaRxDescriptor_t *desc );
#endif

#endif /*_INCADMA_H_*/